@@ -7,16 +7,15 @@ class BaseTask(object):
 
     def __init__(self):
 
-        # Remind user that they forgot to call super().__init__() in
-        # their inherited class __init__().
+        # Define a variable used to remind the user if they forgot to
+        # call super().__init__() in their inherited class __init__().
         self.called_super_in_task_subclass = True
 
         self.addr = None  # String IP Address
         self.task_stop = 0.0  # Data timestamp, float seconds since epoch
         self.task_start = 0.0  # When task was run by a worker
         self.queue_time = 0.0  # When TaskMgr queued task for a worker
-
-        self.worker_loop_delay = 0.00001  # Default 10us worker sleep delay
+        self.worker_loop_delay = 0.00001  # Default 10us worker sleep loop_delay
 
 
     def __hash__(self):
